package com.ningdatech.pmapi.projectdeclared.manage;

import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.ningdatech.basic.function.VUtils;
import com.ningdatech.pmapi.common.helper.UserInfoHelper;
import com.ningdatech.pmapi.projectlib.model.dto.ProjectDTO;
import com.ningdatech.pmapi.projectlib.model.entity.Project;
import com.ningdatech.pmapi.projectlib.service.IProjectService;
import com.ningdatech.pmapi.user.security.auth.model.UserFullInfoDTO;
import com.wflow.workflow.bean.dto.ProcessInstanceUserDto;
import lombok.RequiredArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.springframework.stereotype.Component;
import java.util.Objects;

/**
 * @Classname DefaultDeclaredProjectManage
 *
 * @Description
 * @Date 2023/2/14 17:12
 * @Author PoffyZhang
 */
@Component
@Slf4j
@RequiredArgsConstructor
public class DefaultDeclaredProjectManage {

    private final IProjectService projectService;

    private final UserInfoHelper userInfoHelper;

    //项目名称去重
    public void checkDuplication(ProjectDTO project){
        VUtils.isTrue(projectService.count(Wrappers.lambdaQuery(Project.class)
                        .eq(Project::getProjectName,project.getProjectName())
                        .ne(Objects.nonNull(project.getId()),Project::getId,project.getId())) > 0)
                .throwMessage(String.format("修改失败 此项目名 【%s】 已存在!",project.getProjectName()));
    }

    public ProcessInstanceUserDto buildUser(Long userId){
        UserFullInfoDTO userFullInfo = userInfoHelper.getUserFullInfo(userId);

        VUtils.isTrue(Objects.isNull(userFullInfo))
                .throwMessage("当前登录用户 信息出错!");

        return ProcessInstanceUserDto.builder()
                .userId(String.valueOf(userFullInfo.getUserId()))
                .userName(userFullInfo.getRealName())
                .orgCode(userFullInfo.getOrganizationCode())
                .orgName(userFullInfo.getOrganizationName())
                .build();
    }
}
